﻿using System.Data;
using System.Windows.Forms;

using SampleApplication.Data;

namespace SampleApplication.BaseController
{
	public delegate void UpdateDelegate(DataTable table);
	public delegate void FillByKeysDelegate(DataTable table, DataTableKeys keys);

    public class DetailBaseController<DS, DT, DR> : SampleApplication.BaseController.BaseController
		where DS : DataSet
		where DT : DataTable
		where DR : DataRow
	{
		private DS _dataSet;
		public DS DataSet
		{
			get { return _dataSet; }
			set { _dataSet = value; }
		}

		private DT _dataTable;
		public DT DataTable
		{
			get { return _dataTable; }
			set { _dataTable = value; }
		}

		private BindingSource _bindingSource;
		public BindingSource BindingSource
		{
			get { return _bindingSource; }
			set { _bindingSource = value; }
		}

		private FillByKeysDelegate _fillByKeys;
		public FillByKeysDelegate FillByKeys
		{
			get { return _fillByKeys; }
			set { _fillByKeys = value; }
		}

		private UpdateDelegate _update;
		public UpdateDelegate Update
		{
			get { return _update; }
			set { _update = value; }
		}

        public void InitializeController(DS dataSet, DT dataTable, BindingSource bindingSource, UpdateDelegate update, FillByKeysDelegate fillByKey)
        {
            _dataSet = dataSet;
            _dataTable = dataTable;
            _bindingSource = bindingSource;
            _fillByKeys = fillByKey;
            _update = update;
        }

        public DetailBaseController<DataSet, DataTable, DataRow> GetBaseController()
        {
            DetailBaseController<DataSet, DataTable, DataRow> baseController = new DetailBaseController<DataSet, DataTable, DataRow>();
            baseController.DataSet = _dataSet;
            baseController.DataTable = _dataTable;
            baseController.BindingSource = _bindingSource;
            baseController.FillByKeys = _fillByKeys;
            baseController.Update = _update;
            return baseController;
        }

		public virtual bool AddNew()
		{
			BindingSource.AddNew();
			return true;
		}

		public virtual bool LoadData(DataTableKeys keys)
		{
			if (keys == null) return AddNew();
			FillByKeys(DataTable, keys);
			return true;
		}

		public virtual bool Save()
		{
			BindingSource.EndEdit();
			Update(DataTable);
			return true;
		}

		public virtual bool Delete()
		{
			BindingSource.RemoveCurrent();
			Update(DataTable);
			return true;
		}
	}
}
